<?php
/**
 * Init
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package JFP_WOOCOMMERCE
 */

namespace JFP_WOOCOMMERCE;

/** Exit if accessed directly */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use JFP_WOOCOMMERCE\Customizer\Options;
use JFP_WOOCOMMERCE\Dashboard\System_Dashboard;
use JFP_WOOCOMMERCE\Metabox\Wc_Options;
use JFP_WOOCOMMERCE\Single\Single_Post;
use JFP_WOOCOMMERCE\Upload\Upload;

/**
 * Class Init
 *
 * @package JFP_WOOCOMMERCE
 */
class Init {

	/**
	 * Instance for Init
	 *
	 * @var Init
	 */
	private static $instance;

	/**
	 * Hold instance of object
	 *
	 * @var array
	 */
	private $object;

	/**
	 * Init constructor.
	 */
	private function __construct() {
		$this->load_helper();
		jfp_woocommerce_activation_hook( JFP_WOOCOMMERCE_FILE, [ $this, 'activation_hook' ] );
		jfp_woocommerce_deactivation_hook( JFP_WOOCOMMERCE_FILE, [ $this, 'deactivation_hook' ] );

		$this->setup_option();
		$this->setup_hook();
		$this->object['single']            = new Single_Post();
		$this->object['upload']            = new Upload();
		$this->object['preview_generator'] = new Preview_Generator();
	}

	public static function activation_hook() {
		Preview_Generator::check_mime_type( 'activation' );
	}

	public static function deactivation_hook() {
		Preview_Generator::check_mime_type( 'deactivation' );
	}

	/**
	 * Load helper file
	 */
	private function load_helper() {
		require_once JFP_WOOCOMMERCE_DIR . 'includes/helper.php';
	}

	/**
	 * Setup Init option
	 */
	private function setup_option() {
		$this->object['customizer'] = new Options();
		$this->object['metabox_wc'] = new Wc_Options();
	}

	/**
	 * Setup Init hook
	 */
	private function setup_hook() {

		/** Integration jeg-element */
		add_filter( 'jeg_add_image_size', [ $this, 'register_image_size' ] );
		add_filter( 'jeg_show_default_module_customizer', '__return_false' );

		// epic dashboard integration
		add_filter( 'epic_parent_menu', [ $this, 'epic_parent_menu' ] );
		add_filter( 'epic_admin_menu', [ $this, 'epic_admin_menu' ], 99 );
		add_filter( 'epic_plugin_list', [ $this, 'epic_plugin_list' ] );
		add_filter( 'epic_dashboard_welcome_string', [ $this, 'epic_welcome_string' ] );
		add_filter( 'epic_dashboard_about_string', [ $this, 'epic_about_string' ] );

		add_action( 'admin_enqueue_scripts', [ $this, 'load_backend_assets' ] );
		add_action( 'wp_enqueue_scripts', [ $this, 'load_assets' ] );
	}

	/**
	 * Load backend assets
	 */
	public function load_backend_assets() {
		wp_enqueue_style( 'jfp-single-admin', JFP_WOOCOMMERCE_URL . '/assets/css/admin/main.css', [], JFP_WOOCOMMERCE_VERSION );
	}

	/**
	 * Override parent menu
	 */
	public function epic_parent_menu( $value ) {
		$value['page_title'] = esc_html__( 'Jegtheme', 'jfp-woocommerce' );
		$value['menu_title'] = esc_html__( 'Jegtheme', 'jfp-woocommerce' );

		return $value;
	}

	/**
	 * Override admin menu
	 *
	 * @param $values
	 *
	 * @return mixed
	 */
	public function epic_admin_menu( $values ) {

		foreach ( $values as $index => $value ) {
			if ( 'epic_system' === $value['slug'] ) {
				$values[ $index ]['action'] = [ $this, 'system_status' ];
			}
		}

		return $values;
	}

	/**
	 * System Status
	 */
	public function system_status() {
		$system = new System_Dashboard();
		$system->system_status();
	}

	/**
	 * Get plugin data
	 *
	 * @param $plugins
	 *
	 * @return array
	 */
	public function epic_plugin_list( $plugins ) {
		$plugins[] = [
			'id'      => JFP_WOOCOMMERCE_ID,
			'name'    => JFP_WOOCOMMERCE_NAME,
			'slug'    => JFP_WOOCOMMERCE,
			'version' => JFP_WOOCOMMERCE_VERSION,
			'option'  => 'jeg_font_preview_woocommerce_license',
			'file'    => JFP_WOOCOMMERCE_FILE,
		];

		return $plugins;
	}

	/**
	 * Override welcome string
	 * epic-dashboard lib
	 *
	 * @param $value
	 *
	 * @return string
	 */
	public function epic_welcome_string( $value ) {
		return wp_kses( __( 'Welcome to <strong>Jeg Font Preview Plugin</strong>', 'jfp-woocommerce' ), wp_kses_allowed_html() );
	}

	/**
	 * Override about string
	 * epic-dashboard lib
	 *
	 * @param $value
	 *
	 * @return string
	 */
	public function epic_about_string( $value ) {
		return wp_kses( __( 'Please activate the plugin license to get official support service and automatic update. Read below for additional information.', 'jfp-woocommerce' ), wp_kses_allowed_html() );
	}

	/**
	 * Singleton page for Init class
	 *
	 * @return Init
	 */
	public static function get_instance() {
		if ( null === static::$instance ) {
			static::$instance = new static();
		}

		return static::$instance;
	}

	/**
	 * Helper function to determine whether a plugin is active.
	 *
	 * @param string $plugin_name plugin name, as the plugin-filename.php.
	 *
	 * @return boolean true if the named plugin is installed and active
	 */
	public static function is_plugin_active( $plugin_name ) {

		$active_plugins = (array) get_option( 'active_plugins', [] );

		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, array_keys( get_site_option( 'active_sitewide_plugins', [] ) ) );
		}

		$plugin_filenames = [];

		foreach ( $active_plugins as $plugin ) {

			if ( false !== strpos( $plugin, '/' ) ) {

				/** Normal plugin name (plugin-dir/plugin-filename.php) */
				list( , $filename ) = explode( '/', $plugin );

			} else {

				/** No directory, just plugin file */
				$filename = $plugin;
			}

			$plugin_filenames[] = $filename;
		}

		return in_array( $plugin_name, $plugin_filenames, true );
	}

	/**
	 * Load script
	 */
	public function load_assets() {
		wp_enqueue_style( 'jfp-woocommerce-asset', JFP_WOOCOMMERCE_URL . '/assets/css/style.css', null, JFP_WOOCOMMERCE_VERSION );

		wp_enqueue_script( 'jfp-woocommerce-asset', JFP_WOOCOMMERCE_URL . '/assets/js/script.js', [ 'jquery' ], JFP_WOOCOMMERCE_VERSION, true );
		wp_localize_script( 'jfp-woocommerce-asset', 'jfp', self::localize_script() );
	}

	/**
	 * Localize Script
	 *
	 * @return mixed|void
	 */
	public static function localize_script() {
		$option               = [];
		$option['mime_types'] = Upload::get_list_mime_types();
		$option['ajaxurl']    = admin_url( 'admin-ajax.php' );

		return apply_filters( 'jfp_woocommerce_asset_localize_script', $option );
	}

	/**
	 * Register all image size
	 *
	 * @param array $sizes List of image size.
	 *
	 * @return mixed
	 */
	public function register_image_size( $sizes ) {
		$sizes['120x86'] = [ 120, 86, true, 715 ];

		return $sizes;
	}

	/**
	 * Setup Init option
	 */
	private function setup_upload() {
		$this->object['upload_folder'] = new Upload();
	}
}
