<?php
/**
 * Preview Generator
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package JFP_WOOCOMMERCE
 */

namespace JFP_WOOCOMMERCE;

/** Exit if accessed directly */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Preview_Generator
 *
 * @package JFP_WOOCOMMERCE
 */
class Preview_Generator {

	/**
	 * Preview_Generator constructor.
	 */
	public function __construct() {
		add_action( 'wp_ajax_jeg_font_preview_generator', [ $this, 'ajax_do_action_type' ] );
		add_action( 'wp_ajax_nopriv_jeg_font_preview_generator', [ $this, 'ajax_do_action_type' ] );
	}

	/**
	 * Check mime type
	 *
	 * @param $hook
	 */
	public static function check_mime_type( $hook ) {
		switch ( $hook ) {
			case 'activation':
				$otf        = jfp_get_mime( JFP_WOOCOMMERCE_DIR . '/includes/data/font/Bebas-Regular.otf' );
				$ttf        = jfp_get_mime( JFP_WOOCOMMERCE_DIR . '/includes/data/font/Bebas-Regular.ttf' );
				$mime_types = [
					'ttf' => $ttf,
					'otf' => $otf,
				];
				update_option( 'jfp_mime_types', $mime_types );
				break;
			case 'deactivation':
				delete_option( 'jfp_mime_types' );
				break;
		}
	}

	/**
	 * @param $text
	 * @param $size
	 * @param $id
	 * @param string $background_color
	 * @param string $font_color
	 * @param $width
	 * @param int    $background_opacity
	 *
	 * @return false|string
	 */
	private function create_image( $text, $size, $id, $background_color = '#FFFFFF', $font_color = '#000000', $width = null, $background_opacity ) {
		// $background_opacity = $background_opacity < 100 ? 127 : 0; // if div render has background
		$background_opacity = 127 - 127 * $background_opacity / 100;
		$background_color   = jfp_convert_hex_to_rgb( $background_color, (int) $background_opacity );
		$font_color         = jfp_convert_hex_to_rgb( $font_color );
		$font               = get_attached_file( $id );
		$width              = $width !== null ? $width : 680;
		$gd_size            = imagecreatetruecolor( $width, 240 );
		imagealphablending( $gd_size, true );
		imagesavealpha( $gd_size, true );
		$background_coloring = isset( $background_color['a'] ) ? imagecolorallocatealpha( $gd_size, $background_color['r'], $background_color['g'], $background_color['b'], $background_color['a'] ) : imagecolorallocate( $gd_size, $background_color['r'], $background_color['g'], $background_color['b'] );
		imagefill( $gd_size, 0, 0, 2147483647 ); // 2147483647 mean transparent
		$new_gd_size = $this->fit_image_to_text( $gd_size, $text, $size, 0, $font, $background_coloring, $font_color );
		imagedestroy( $gd_size );
		ob_start();
		imagepng( $new_gd_size );
		$image_base64 = ob_get_clean();
		imagedestroy( $new_gd_size );

		return base64_encode( $image_base64 );
	}

	/**
	 * Fit Image size to Text Size
	 *
	 * @param resource|boolean $gd_size image identifier size.
	 * @param string           $text Input text.
	 * @param int              $size Font Size.
	 * @param int              $angle Angle of text.
	 * @param string           $font The name of the TrueType font file.
	 * @param string           $background_coloring Generated background coloring.
	 * @param array            $font_color Font Color
	 *
	 * @return resource
	 */
	private function fit_image_to_text( $gd_size, $text, $size, $angle, $font, $background_coloring, $font_color ) {
		/** Set the environment variable for GD */
		putenv( 'GDFONTPATH=' . ABSPATH );

		$rect          = imageftbbox( $size, $angle, $font, $text );
		$min_x         = min( [ $rect[0], $rect[2], $rect[4], $rect[6] ] );
		$max_x         = max( [ $rect[0], $rect[2], $rect[4], $rect[6] ] );
		$min_y         = min( [ $rect[1], $rect[3], $rect[5], $rect[7] ] );
		$max_y         = max( [ $rect[1], $rect[3], $rect[5], $rect[7] ] );
		$target_width  = $max_x - $min_x;
		$target_height = $max_y - $min_y;
		$src_width     = imagesx( $gd_size );
		$src_height    = imagesy( $gd_size );

		/** Recovering new source image size respecting its aspect ratio */

		$src_ratio    = $src_width / $src_height;
		$target_ratio = $target_width / $target_height;
		if ( $src_width <= $target_width && $src_height <= $target_height ) {
			$img_target_width  = $src_width;
			$img_target_height = $src_height;
		} elseif ( $target_ratio > $src_ratio ) {
			$img_target_width  = (int) ( $target_height * $src_ratio );
			$img_target_height = $target_height;
		} else {
			$img_target_width  = $target_width;
			$img_target_height = (int) ( $target_height / $src_ratio );
		}
		$target_gd_size = imagecreatetruecolor( $src_width, $target_height );
		imagealphablending( $target_gd_size, true );
		imagesavealpha( $target_gd_size, true );
		imagefill( $target_gd_size, 0, 0, $background_coloring );
		imagecopyresampled( $target_gd_size, $gd_size, 0, 0, 0, 0, $img_target_width, $img_target_height, $src_width, $src_height );
		imagefttext( $target_gd_size, $size, $angle, abs( $min_x ), abs( $min_y ), imagecolorallocate( $target_gd_size, $font_color['r'], $font_color['g'], $font_color['b'] ), $font, $text );

		return $target_gd_size;
	}

	/**
	 * Generate image
	 *
	 * @param $text
	 * @param $size
	 * @param $font
	 * @param null   $width
	 * @param string $background_color
	 * @param string $font_color
	 * @param string $background_opacity
	 *
	 * @return array
	 */
	public function generate_single_image_font( $text, $size, $font, $width = null, $background_color = '#FFFFFF', $font_color = '#000000', $background_opacity ) {
		$result = [
			'id'    => $font['ttf_file'],
			'name'  => $font['font_name'],
			'image' => $this->create_image( $text, (int) $size, $font['ttf_file'], $background_color, $font_color, $width, $background_opacity ),
		];

		return $result;
	}

	public function ajax_do_action_type() {
		if ( isset( $_POST['type'], $_POST['nonce'] ) ) {
			$nonce = empty( $_POST['nonce'] ) ? false : wp_verify_nonce( sanitize_key( $_POST['nonce'] ), 'jfp-preview-generator-nonce' );
			if ( $nonce ) {
				$result = [];
				switch ( $_POST['type'] ) {
					case 'create_image':
						$text               = isset( $_POST['text'] ) ? $_POST['text'] : '';
						$font               = isset( $_POST['font'] ) ? $_POST['font'] : [];
						$size               = isset( $_POST['font_size'] ) ? $_POST['font_size'] : 24;
						$font_color         = isset( $_POST['font_color'] ) ? $_POST['font_color'] : '#000000';
						$background_color   = isset( $_POST['background_color'] ) ? $_POST['background_color'] : '#FFFFFF';
						$background_opacity = isset( $_POST['background_opacity'] ) ? $_POST['background_opacity'] : 100;
						$width              = isset( $_POST['width'] ) ? $_POST['width'] : null;
						if ( ! empty( $font ) && ! empty( $text ) ) {
							$result = $this->generate_single_image_font( stripslashes( $text ), (int) $size, $font, $width, $background_color, $font_color, $background_opacity );
						}
						wp_send_json_success( $result );
						break;
					default:
						wp_send_json_error(
							[
								'response' => 0,
								'message'  => esc_html__( 'Internal Server Error!', 'jfp-woocommerce' ),
							]
						);
						break;
				}
			}
		}
		wp_send_json_error(
			[
				'response' => 0,
				'message'  => esc_html__( 'Internal Server Error!', 'jfp-woocommerce' ),
			]
		);
	}
}
