<?php
/**
 * Options Abstract
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package JFP_WOOCOMMERCE\Metabox
 */

namespace JFP_WOOCOMMERCE\Metabox;

/** Exit if accessed directly */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Jeg\Form\Form_Meta_Box;
use JFP_WOOCOMMERCE\Upload\Upload;

/**
 * Class Options
 *
 * @package JFP_WOOCOMMERCE\Metabox
 */
class Options {

	/**
	 * ID of Metabox
	 *
	 * @var string
	 */
	private static $id = 'jfp-woocommerce-settings';

	/**
	 * Data of Metabox Post Type Setting
	 *
	 * @var string
	 */
	private $post_type;

	/**
	 * Options constructor.
	 *
	 * @param string|array $post_type Data of Metabox Post Type Setting.
	 */
	protected function __construct( $post_type = '' ) {
		$this->post_type = $post_type;
		add_action( 'plugins_loaded', [ $this, 'register_metabox' ] );
	}

	/**
	 * Singleton page of Wc_Options class
	 *
	 * @return Options
	 */
	public static function get_instance() {
		if ( null === static::$instance ) {
			static::$instance = new static();
		}

		return static::$instance;
	}

	/**
	 * Register metabox
	 */
	public function register_metabox() {
		$option = [
			'id'        => self::$id,
			'title'     => esc_html__( 'Font Preview Setting', 'jfp-woocommerce' ),
			'post_type' => $this->post_type,
			'type'      => 'normal',
			'fields'    => $this->metabox_fields(),
			'segments'  => $this->metabox_segments(),
		];

		new Form_Meta_Box( $option );
	}

	/**
	 * Set Metabox field
	 */
	protected function metabox_fields() {
		$mime_types = implode( ',', Upload::get_list_mime_types());

		$fields = [];

		$fields['preview'] = [
			'segment'     => 'default',
			'type'        => 'checkbox',
			'title'       => esc_attr__( 'Enable Font Preview', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Enable this option to show the font preview for the current product.', 'jfp-woocommerce' ),
			'default'     => false,
		];

		$fields['placeholder'] = [
			'segment'     => 'default',
			'type'        => 'text',
			'title'       => esc_attr__( 'Placeholder', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Fill this option to set the font preview placeholder', 'jfp-woocommerce' ),
			'default'     => '',
			'dependency'  => [
				[
					'field'    => 'preview',
					'operator' => '==',
					'value'    => true,
				],
			],
		];

		$fields['font_color'] = [
			'segment'     => 'default',
			'type'        => 'color',
			'title'       => esc_attr__( 'Font Color', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Choose initial font color for the font preview.', 'jfp-woocommerce' ),
			'default'     => '#000000',
			'dependency'  => [
				[
					'field'    => 'preview',
					'operator' => '==',
					'value'    => true,
				],
			],
		];

		$fields['background_color'] = [
			'segment'     => 'default',
			'type'        => 'color',
			'title'       => esc_attr__( 'Background Color', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Choose initial background color for the font preview.', 'jfp-woocommerce' ),
			'default'     => '#FFFFFF',
			'dependency'  => [
				[
					'field'    => 'preview',
					'operator' => '==',
					'value'    => true,
				],
			],
		];

		$fields['background_opacity'] = [
			'segment'     => 'default',
			'type'        => 'slider',
			'title'       => esc_attr__( 'Background Opacity', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Set opacity property for the background color in percent (%).', 'jfp-woocommerce' ),
			'options'     => [
				'min'  => 0,
				'max'  => 100,
				'step' => 1,
			],
			'default'     => 100,
			'dependency'  => [
				[
					'field'    => 'preview',
					'operator' => '==',
					'value'    => true,
				],
			],
		];

		$fields['font_size'] = [
			'segment'     => 'default',
			'type'        => 'slider',
			'title'       => esc_attr__( 'Font Size', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Set initial font size for the font preview (px).', 'jfp-woocommerce' ),
			'options'     => [
				'min'  => 4,
				'max'  => 240,
				'step' => 4,
			],
			'default'     => 24,
			'dependency'  => [
				[
					'field'    => 'preview',
					'operator' => '==',
					'value'    => true,
				],
			],
		];

		$fields['min_font_size'] = [
			'segment'     => 'default',
			'type'        => 'slider',
			'title'       => esc_attr__( 'Minimal Font Size', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Set minimal font size for the font preview (px).', 'jfp-woocommerce' ),
			'options'     => [
				'min'  => 4,
				'max'  => 240,
				'step' => 4,
			],
			'default'     => 4,
			'dependency'  => [
				[
					'field'    => 'preview',
					'operator' => '==',
					'value'    => true,
				],
			],
		];

		$fields['max_font_size'] = [
			'segment'     => 'default',
			'type'        => 'slider',
			'title'       => esc_attr__( 'Maximal Font Size', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Set maximal font size for the font preview (px).', 'jfp-woocommerce' ),
			'options'     => [
				'min'  => 4,
				'max'  => 240,
				'step' => 4,
			],
			'default'     => 240,
			'dependency'  => [
				[
					'field'    => 'preview',
					'operator' => '==',
					'value'    => true,
				],
			],
		];

		$fields['fonts'] = [
			'segment'     => 'default',
			'type'        => 'repeater',
			'title'       => esc_attr__( 'Fonts', 'jfp-woocommerce' ),
			'description' => esc_attr__( 'Add font lists', 'jfp-woocommerce' ),
			'default'     => [],
			'choices'     => [
				'limit' => '',
			],
			'row_label'   => [
				'type'  => 'text',
				'value' => esc_attr__( 'Font Variations', 'jfp-woocommerce' ),
				'field' => false,
			],
			'fields'      => [
				'font_name' => [
					'type'        => 'text',
					'label'       => esc_attr__( 'Font Name', 'jfp-woocommerce' ),
					'description' => esc_attr__( 'Please fill your font name. You can put same font name on font uploader in case you have several types of font (bold, italic or other).', 'jfp-woocommerce' ),
					'default'     => '',
					'id'          => 'font_name',
				],
				'ttf_file'  => [
					'type'        => 'upload',
					'label'       => esc_attr__( 'OTF/TTF File', 'jfp-woocommerce' ),
					'description' => esc_attr__( 'TrueType Fonts', 'jfp-woocommerce' ),
					'default'     => '',
					'mime_type'   => $mime_types,
					'id'          => 'ttf_file',
				],
			],
			'dependency'  => [
				[
					'field'    => 'preview',
					'operator' => '==',
					'value'    => true,
				],
			],
		];

		return $fields;
	}

	/**
	 * Set Metabox Segments
	 */
	protected function metabox_segments() {
		$segment = [];

		$segment['default'] = [
			'name'     => esc_html__( 'Default Segment', 'jfp-woocommerce' ),
			'priority' => 1,
		];

		return $segment;
	}
}
