<?php
/**
 * Upload Abstract
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package JFP_WOOCOMMERCE\Upload
 */

namespace JFP_WOOCOMMERCE\Upload;

/** Exit if accessed directly */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class Upload
 *
 * @package JFP_WOOCOMMERCE\Upload
 */
class Upload {

	/**
	 * TTF and OTF Font mime type
	 *
	 * @var array
	 */
	private static $mime_types = [
		[ 'ttf' => 'application/x-font-truetype' ],
		[ 'ttf' => 'application/x-font-ttf' ],
		[ 'ttf' => 'font/ttf' ],
		[ 'ttf' => 'application/octet-stream' ],
		[ 'otf' => 'application/x-font-opentype' ],
		[ 'otf' => 'application/x-font-otf' ],
		[ 'otf' => 'font/otf' ],
		[ 'otf' => 'application/octet-stream' ]
	];
	/**
	 * Font Extension
	 *
	 * @var array
	 */
	private $font_extension = [ 'ttf', 'otf' ];

	/**
	 * Upload constructor.
	 */
	public function __construct() {
		self::setup_mime_type();
		add_filter( 'wp_check_filetype_and_ext', [ $this, 'add_multiple_mime_types' ], 99, 3 );
		$override_upload = jfp_get_option( 'override_upload_font_location', false );
		if ( $override_upload ) {
			/** Change directory upload */
			add_action( 'admin_init', [ $this, 'jfp_woocommerce_change_downloads_upload_dir' ], 2000 );
			add_action( 'admin_init', [ $this, 'jfp_woocommerce_create_protection_files' ] );
		}
	}

	/**
	 * Get allowed mime types extension
	 *
	 * @return array
	 */
	public static function get_allowed_mime_types() {
		self::setup_mime_type();

		return self::$mime_types;
	}

	/**
	 * Merge detected mime type from server
	 */
	public static function setup_mime_type() {
		$mime_types = get_option( 'jfp_mime_types', [] );
		if ( isset( $mime_types['otf'] ) && isset( $mime_types['ttf'] ) ) {
			$new_mime_types = [];
			$ttf            = [ 'ttf' => $mime_types['ttf'] ];
			$otf            = [ 'otf' => $mime_types['otf'] ];
			if ( ! jfp_in_array_key( $mime_types['ttf'], 'ttf', self::$mime_types, true ) ) {
				$new_mime_types[] = $ttf;
			}
			if ( ! jfp_in_array_key( $mime_types['otf'], 'otf', self::$mime_types, true ) ) {
				$new_mime_types[] = $otf;
			}

			self::$mime_types = array_merge( self::$mime_types, $new_mime_types );
		}
	}

	/**
	 * Multiple mime types in a extension
	 *
	 * @param $check
	 * @param $file
	 * @param $filename
	 *
	 * @return array
	 */
	public function add_multiple_mime_types( $check, $file, $filename ) {
		if ( empty( $check['ext'] ) && empty( $check['type'] ) ) {
			foreach ( self::get_allowed_mime_types() as $mime ) {
				remove_filter( 'wp_check_filetype_and_ext', [ $this, 'add_multiple_mime_types' ], 99 );
				$mime_filter = function ( $mimes ) use ( $mime ) {
					return array_merge( $mimes, $mime );
				};

				add_filter( 'upload_mimes', $mime_filter, 99 );
				$check = wp_check_filetype_and_ext( $file, $filename, $mime );
				remove_filter( 'upload_mimes', $mime_filter, 99 );
				add_filter( 'wp_check_filetype_and_ext', [ $this, 'add_multiple_mime_types' ], 99, 3 );
				if ( ! empty( $check['ext'] ) || ! empty( $check['type'] ) ) {
					return $check;
				}
			}
		}

		return $check;
	}

	/**
	 * List of mime type
	 *
	 * @return array
	 */
	public static function get_list_mime_types() {
		$mime_types = [];
		foreach ( self::get_allowed_mime_types() as $val ) {
			foreach ( $val as $ext => $mime_type ) {
				if ( ! in_array( $mime_type, $mime_types, true ) ) {
					$mime_types[] = $mime_type;
				}
			}
		}
		return $mime_types;
	}

	/**
	 * Change Downloads Upload Directory
	 *
	 * @return void
	 * @since 1.0
	 */
	public function jfp_woocommerce_change_downloads_upload_dir() {
		if ( isset( $_POST['upload-attachment'] ) && $_POST['upload-attachment'] === 'upload-attachment' ) {
			$file_type      = wp_check_filetype( $_POST['name'] );
			$file_mime_type = $file_type['type'];
			$file_extension = strtolower( $file_type['ext'] );
			$mime_types     = self::get_list_mime_types();
			if ( in_array( $file_mime_type, $mime_types, true ) && in_array( $file_extension, $this->font_extension, true ) ) {
				$this->jfp_woocommerce_create_protection_files( true );
				add_filter( 'upload_dir', [ $this, 'jfp_woocommerce_set_upload_dir' ] );
			}
		}
	}

	/**
	 * Creates blank index.php and .htaccess files
	 *
	 * This function runs approximately once per month in order to ensure all folders
	 * have their necessary protection files
	 *
	 * @param bool $force
	 * @param bool $method
	 *
	 * @since 1.1.5
	 */
	public function jfp_woocommerce_create_protection_files( $force = false, $method = false ) {
		if ( false === get_transient( 'jfp_woocommerce_check_protection_files' ) || $force ) {

			$upload_path = $this->jfp_woocommerce_get_upload_dir();

			// Make sure the /jfp folder is created
			wp_mkdir_p( $upload_path );

			// Top level .htaccess file
			$rules = $this->jfp_woocommerce_get_htaccess_rules( $method );
			if ( $this->jfp_woocommerce_htaccess_exists() ) {
				$contents = @file_get_contents( $upload_path . '/.htaccess' );
				if ( $contents !== $rules || ! $contents ) {
					// Update the .htaccess rules if they don't match
					@file_put_contents( $upload_path . '/.htaccess', $rules );
				}
			} elseif ( $this->wp_is_writable( $upload_path ) ) {
				// Create the file if it doesn't exist
				@file_put_contents( $upload_path . '/.htaccess', $rules );
			}

			// Top level blank index.php
			if ( ! file_exists( $upload_path . '/index.php' ) && $this->wp_is_writable( $upload_path ) ) {
				@file_put_contents( $upload_path . '/index.php', '<?php' . PHP_EOL . '// Silence is golden.' );
			}

			// Now place index.php files in all sub folders
			$folders = $this->jfp_woocommerce_scan_folders( $upload_path );
			foreach ( $folders as $folder ) {
				// Create index.php, if it doesn't exist
				if ( ! file_exists( $folder . 'index.php' ) && $this->wp_is_writable( $folder ) ) {
					@file_put_contents( $folder . 'index.php', '<?php' . PHP_EOL . '// Silence is golden.' );
				}
			}
			// Check for the files once per day
			set_transient( 'jfp_woocommerce_check_protection_files', true, 3600 * 24 );
		}
	}

	/**
	 * Retrieve the absolute path to the file upload directory without the trailing slash
	 *
	 * @return string $path Absolute path to the JFP_WOOCOMMERCE upload directory
	 * @since  1.8
	 */
	public function jfp_woocommerce_get_upload_dir() {
		$wp_upload_dir = wp_upload_dir();
		wp_mkdir_p( $wp_upload_dir['basedir'] . '/jfp' );
		$path = $wp_upload_dir['basedir'] . '/jfp';

		return apply_filters( 'jfp_woocommerce_get_upload_dir', $path );
	}

	/**
	 * Retrieve the .htaccess rules to wp-content/uploads/jfp/
	 *
	 * @param bool $method
	 *
	 * @return mixed|void The htaccess rules
	 * @since 1.6
	 */
	public function jfp_woocommerce_get_htaccess_rules( $method = false ) {

		if ( empty( $method ) ) {
			$method = 'direct';
		}

		switch ( $method ) :

			case 'redirect':
				// Prevent directory browsing
				$rules = 'Options -Indexes';
				break;

			case 'direct':
			default:
				$rules = "Options -Indexes\n";
				$rules .= "deny from all\n";
				break;

		endswitch;
		$rules = apply_filters( 'jfp_woocommerce_protected_directory_htaccess_rules', $rules, $method );

		return $rules;
	}

	/**
	 * Checks if the .htaccess file exists in wp-content/uploads/jfp
	 *
	 * @return bool
	 * @since 1.8
	 */
	public function jfp_woocommerce_htaccess_exists() {
		$upload_path = $this->jfp_woocommerce_get_upload_dir();

		return file_exists( $upload_path . '/.htaccess' );
	}

	/**
	 * Determine if a directory is writable.
	 *
	 * This function is used to work around certain ACL issues
	 * in PHP primarily affecting Windows Servers.
	 *
	 * @param string $path
	 *
	 * @return bool
	 * @see win_is_writable()
	 *
	 * @since 3.6.0
	 */
	public function wp_is_writable( $path ) {
		if ( 'WIN' === strtoupper( substr( PHP_OS, 0, 3 ) ) ) {
			return win_is_writable( $path );
		}

		return @is_writable( $path );
	}

	/**
	 * Scans all folders inside of /uploads/jfp
	 *
	 * @return array $return List of files inside directory
	 * @since 1.1.5
	 */
	public function jfp_woocommerce_scan_folders( $path = '', $return = [] ) {
		$path  = $path == '' ? dirname( __FILE__ ) : $path;
		$lists = @scandir( $path );

		if ( ! empty( $lists ) ) {
			foreach ( $lists as $f ) {
				if ( is_dir( $path . DIRECTORY_SEPARATOR . $f ) && $f != '.' && $f != '..' ) {
					if ( ! in_array( $path . DIRECTORY_SEPARATOR . $f, $return, true ) ) {
						$return[] = trailingslashit( $path . DIRECTORY_SEPARATOR . $f );
					}

					$this->jfp_woocommerce_scan_folders( $path . DIRECTORY_SEPARATOR . $f, $return );
				}
			}
		}

		return $return;
	}

	/**
	 * Set Upload Directory
	 *
	 * @return array Upload directory information
	 * @since 1.0
	 */
	public function jfp_woocommerce_set_upload_dir( $upload ) {

		// Override the year / month being based on the post publication date, if year/month organization is enabled
		if ( get_option( 'uploads_use_yearmonth_folders' ) ) {
			// Generate the yearly and monthly dirs
			$time             = current_time( 'mysql' );
			$y                = substr( $time, 0, 4 );
			$m                = substr( $time, 5, 2 );
			$upload['subdir'] = "/$y/$m";
		}

		$upload['subdir'] = '/jfp' . $upload['subdir'];
		$upload['path']   = $upload['basedir'] . $upload['subdir'];
		$upload['url']    = $upload['baseurl'] . $upload['subdir'];

		return $upload;
	}

}
