<?php
/**
 * Misc Functions
 *
 * @author Jegtheme
 * @since 1.0.0
 * @package JFP_WOOCOMMERCE
 */

/** Exit if accessed directly */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Get option for Font Preview
 *
 * @param string $setting Setting name.
 * @param mixed $default Default option.
 *
 * @return mixed
 */
if ( ! function_exists( 'jfp_get_option' ) ) {
	function jfp_get_option( $setting, $default = null ) {
		$options = get_option( 'jfp_option', [] );
		$value   = $default;

		if ( isset( $options[ $setting ] ) ) {
			$value = $options[ $setting ];
		}

		return $value;
	}
}

if ( ! function_exists( 'jfp_woocommerce_activation_hook' ) ) {
	/**
	 * Register activation hook
	 *
	 * @param string $file Plugin file path.
	 */
	function jfp_woocommerce_activation_hook( $file, $function ) {
		register_activation_hook( $file, $function );
	}
}

if ( ! function_exists( 'jfp_in_array_key' ) ) {
	/**
	 * In array recursive with key
	 *
	 * @param $needle
	 * @param $needle_key
	 * @param $haystack
	 * @param bool       $strict
	 *
	 * @return bool
	 */
	function jfp_in_array_key( $needle, $needle_key, $haystack, $strict = false ) {
		foreach ( $haystack as $type => $mime_type ) {
			if ( ( ( $strict ? $mime_type === $needle : $mime_type == $needle ) && ( $strict ? $type === $needle_key : $type == $needle_key ) ) || ( is_array( $mime_type ) && jfp_in_array_key( $needle, $needle_key, $mime_type, $strict ) ) ) {
				return true;
			}
		}

		return false;
	}
}

if ( ! function_exists( 'jfp_woocommerce_deactivation_hook' ) ) {
	/**
	 * Register deactivation hook
	 *
	 * @param string $file Plugin file path.
	 */
	function jfp_woocommerce_deactivation_hook( $file, $function ) {
		register_deactivation_hook( $file, $function );
	}
}

if ( ! function_exists( 'jfp_woocommerce_get_template_part' ) ) {
	/**
	 * @param $slug
	 * @param null $name
	 */
	function jfp_woocommerce_get_template_part( $slug, $name = null ) {
		do_action( "jfp_woocommerce_get_template_part_{$slug}", $slug, $name );
		$templates = [];
		if ( isset( $name ) ) {
			$templates[] = "{$slug}-{$name}.php";
		}
		$templates[] = "{$slug}.php";
		jfp_woocommerce_get_template_path( $templates, true, false );
	}
}

if ( ! function_exists( 'jfp_woocommerce_preview_is_active' ) ) {
	/**
	 * @param int $post_id
	 *
	 * @return bool
	 */
	function jfp_woocommerce_preview_is_active( $post_id ) {
		return jeg_metabox( 'jfp-woocommerce-settings.preview', null, (int) $post_id ) === '1';
	}
}

if ( ! function_exists( 'jfp_woocommerce_get_template_path' ) ) {
	/**
	 * @param $template_names
	 * @param bool           $load
	 * @param bool           $require_once
	 *
	 * @return string
	 */
	function jfp_woocommerce_get_template_path( $template_names, $load = false, $require_once = true ) {
		$located = '';
		foreach ( (array) $template_names as $template_name ) {
			if ( ! $template_name ) {
				continue;
			}
			/* search file within the JFP_WOOCOMMERCE_DIR only */
			if ( file_exists( JFP_WOOCOMMERCE_DIR . $template_name ) ) {
				$located = JFP_WOOCOMMERCE_DIR . $template_name;
				break;
			}
		}
		if ( $load && '' !== $located ) {
			load_template( $located, $require_once );
		}

		return $located;
	}
}

if ( ! function_exists( 'jfp_get_file_name' ) ) {
	add_action( 'wp_ajax_jfp_get_file_name', 'jfp_get_file_name' );
	/**
	 * Get file ajax
	 */
	function jfp_get_file_name() {
		if ( isset( $_REQUEST['nonce'], $_REQUEST['id'] ) && wp_verify_nonce( sanitize_key( $_REQUEST['nonce'] ), 'jfp_get_attachment_option' ) ) {
			$result = jfp_get_attachment_option( $_REQUEST['id'] );

			wp_send_json_success( $result );
		}
	}
}

if ( ! function_exists( 'jfp_get_attachment_option' ) ) {
	/**
	 * Get attachment title by ID
	 *
	 * @param string $value ID separated with comma.
	 *
	 * @return array
	 */
	function jfp_get_attachment_option( $value = null ) {
		$result = [];

		if ( ! empty( $value ) ) {
			$result = [
				'value' => $value,
				'text'  => basename( wp_get_attachment_url( $value ) ),
			];
		}

		return $result;
	}
}

if ( ! function_exists( 'jfp_convert_hex_to_rgb' ) ) {
	/**
	 * Convert Hex to RGB
	 *
	 * @param string $hex The Hex Color.
	 * @param string $alpha Transparency.
	 *
	 * @return array
	 */
	function jfp_convert_hex_to_rgb( $hex = '#000000', $alpha = null ) {
		$hex      = str_replace( '#', '', $hex );
		$length   = strlen( $hex );
		$rgb['r'] = hexdec( 6 === $length ? substr( $hex, 0, 2 ) : ( 3 === $length ? str_repeat( substr( $hex, 0, 1 ), 2 ) : 0 ) );
		$rgb['g'] = hexdec( 6 === $length ? substr( $hex, 2, 2 ) : ( 3 === $length ? str_repeat( substr( $hex, 1, 1 ), 2 ) : 0 ) );
		$rgb['b'] = hexdec( 6 === $length ? substr( $hex, 4, 2 ) : ( 3 === $length ? str_repeat( substr( $hex, 2, 1 ), 2 ) : 0 ) );
		if ( $alpha !== null ) {
			if ( is_int( (int) $alpha ) || is_float( (float) $alpha ) ) {
				$rgb['a'] = $alpha;
			}
		}

		return $rgb;
	}
}

if ( ! function_exists( 'jfp_get_mime' ) ) {
	/**
	 * Get mime type file
	 *
	 * @param string $filename  File name and path file.
	 *
	 * @return mixed|string
	 */
	function jfp_get_mime( $filename ) {
		if ( function_exists( 'mime_content_type' ) ) {
			return mime_content_type( $filename );
		}

		if ( function_exists( 'finfo_open' ) ) {
			$finfo    = finfo_open( FILEINFO_MIME );
			$mimetype = finfo_file( $finfo, $filename );
			finfo_close( $finfo );

			return $mimetype;
		} else {
			return 'application/octet-stream';
		}
	}
}

/** Print Translation */

if ( ! function_exists( 'jeg_print_translation' ) ) {
	function jeg_print_translation( $string, $domain, $name ) {
		do_action( 'jeg_return_translation', $string, $domain, $name );
	}
}


if ( ! function_exists( 'jeg_print_main_translation' ) ) {
	add_action( 'jeg_print_translation', 'jeg_print_main_translation', 10, 2 );

	function jeg_print_main_translation( $string, $domain ) {
		call_user_func_array( 'esc_html_e', [ $string, $domain ] );
	}
}

/** Return Translation */

if ( ! function_exists( 'jeg_return_translation' ) ) {
	function jeg_return_translation( $string, $domain, $name, $escape = true ) {
		return apply_filters( 'jeg_return_translation', $string, $domain, $name, $escape );
	}
}

if ( ! function_exists( 'jeg_return_main_translation' ) ) {
	add_filter( 'jeg_return_translation', 'jeg_return_main_translation', 10, 4 );

	function jeg_return_main_translation( $string, $domain, $name, $escape = true ) {
		if ( $escape ) {
			return call_user_func_array( 'esc_html__', [ $string, $domain ] );
		}

		return call_user_func_array( '__', [ $string, $domain ] );

	}
}
